/*
 *  Proxy-applet.cs: a tray applet for direct access to proxy preferences.
 *  Copyright 2003 Dag Wieers <dag@wieers.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Diagnostics;

using Gnome;
using GnomeSharp;
using Gtk;
using GtkSharp;
using Glade;

using Egg;

public class ProxyTrayIconApplication {
	public static void Main(string[] args) {
		Program program = new Program ("Proxy Applet", "0.2.3", Gnome.Modules.UI, args);
		TrayIcon tray = new ProxyTrayIcon("Proxy Applet");
		tray.ShowAll();
		program.Run();
	}
}

public class ProxyTrayIcon : TrayIcon {
	const string NAME = "Proxy Applet";
	const string VERSION = "0.2.3";
	private Gdk.Pixbuf ICON = new Gdk.Pixbuf (null, "pixmaps/proxy-applet.png");

	private EventBox eb = new EventBox();
	private GConf.Client gc1 = new GConf.Client();
	private GConf.Client gc2 = new GConf.Client();
	private GConf.Client gc3 = new GConf.Client();
	private GConf.Client gc4 = new GConf.Client();
	private Tooltips traytooltip = new Tooltips();
	[Glade.Widget] Menu leftmenu, rightmenu;
	[Glade.Widget] RadioMenuItem no_proxy, manual_proxy, auto_proxy;
	[Glade.Widget] MenuItem madd, mprofiles;
//	[Glade.Widget] SeparatorMenuItem mseparator1;

	public ProxyTrayIcon (string name) : base(name) {
		Glade.XML gxml = new Glade.XML(null, "proxy-applet.glade", null, null);
		gxml.Autoconnect(this);

		rightmenu.Remove(madd);
		rightmenu.Remove(mprofiles);

		resetProxyPref();
		getProxyPref(no_proxy, null);

		// FIXME: resize icon according to panel if notification area allows this
		eb.Add (new Image(ICON));
		eb.ButtonPressEvent += new ButtonPressEventHandler(on_icon_click);

		this.Add(eb);
	}

	// Reset to default GConf proxy values, if they don't exist and add event handlers.
	private void resetProxyPref() {
		// FIXME: Do I really need to have a seperate GConf.Client for each EventHandler ?
		gc1.AddNotify("/system/proxy/mode", new GConf.NotifyEventHandler(getProxyPref));
		try {
			gc1.Get("/system/proxy/mode");
		} catch(GConf.NoSuchKeyException) {
			gc1.Set("/system/proxy/mode", "none");
		}
		gc2.AddNotify("/system/http_proxy/host", new GConf.NotifyEventHandler(getProxyPref));
		try {
			gc2.Get("/system/http_proxy/host");
		} catch(GConf.NoSuchKeyException) {
			gc2.Set("/system/http_proxy/host", "proxy");
		}
		gc3.AddNotify("/system/http_proxy/port", new GConf.NotifyEventHandler(getProxyPref));
		try {
			gc3.Get("/system/http_proxy/port");
		} catch(GConf.NoSuchKeyException) {
			gc3.Set("/system/http_proxy/port", "8080");
		}
		gc4.AddNotify("/system/proxy/autoconfig_url", new GConf.NotifyEventHandler(getProxyPref));
		try {
			gc4.Get("/system/proxy/autoconfig_url");
		} catch(GConf.NoSuchKeyException) {
			gc4.Set("/system/proxy/autoconfig_url", "http://pac:8080/");
		}
	}

	// Get GConf proxy settings and update visuals.
	private void getProxyPref(object o, GConf.NotifyEventArgs args) {
		switch((string) gc1.Get("/system/proxy/mode")) {
			case "none":
				no_proxy.Active = true;
				traytooltip.SetTip(eb, "Direct internet connection\nNo proxy", "");
				break;
			case "manual":
				manual_proxy.Active = true;
				string host = (string) gc2.Get("/system/http_proxy/host");
				int port = (int) gc3.Get("/system/http_proxy/port");
				traytooltip.SetTip(eb, "Manual proxy configuration\nProxy: " + host + ":" + port, "");
				break;
			case "auto":
				auto_proxy.Active = true;
				string url = (string) gc4.Get("/system/proxy/autoconfig_url");
				traytooltip.SetTip(eb, "Automatic proxy configuration\nURL: " + url, "");
				break;
//			default:
			// FIXME: If none of the above, warn user !
		}
	}

	// Change GConf proxy values when selected.
	private void on_proxy_change(object o, EventArgs args) {
		if (o == no_proxy) 
			gc1.Set("/system/proxy/mode", "none");
		else if (o == manual_proxy)
			gc1.Set("/system/proxy/mode", "manual");
		else if (o == auto_proxy)
			gc1.Set("/system/proxy/mode", "auto");
	}

	private void on_icon_click(object o, ButtonPressEventArgs args) {
		if (args.Event.button == 3) { 
			rightmenu.ShowAll();
			rightmenu.Popup(null, null, null, IntPtr.Zero, args.Event.button, args.Event.time);
		} else if (args.Event.button == 1) {
			leftmenu.ShowAll();
			leftmenu.Popup(null, null, null, IntPtr.Zero, args.Event.button, args.Event.time);
		}
	}

	private void on_madd_activate(object o, EventArgs args) { }
	private void on_mprofiles_activate(object o, EventArgs args) { }

	private void on_mproxy_config_activate(object o, EventArgs args) {
		Process.Start("gnome-network-preferences");
	}

	private void on_mhelp_activate(object o, EventArgs args) {
		string browser;
		GConf.Client gc = new GConf.Client();
		try {
			browser = (string) gc.Get("/desktop/gnome/applications/browser/exec");
		} catch(GConf.NoSuchKeyException) {
			browser = "galeon";
		}
		Process.Start(browser + " 'http://dag.wieers.com/home-made/gnome-applets/#proxy-applet'");
	}

	private void on_mabout_activate(object o, EventArgs args) {
		// Due to bug in Gtk# cannot use Glade About.Dialog ;-(
		Gdk.Pixbuf logo = new Gdk.Pixbuf (null, "pixmaps/proxy-applet-48x48.png");
		string copy = "Copyright (C) 2003 Dag Wieers <dag@wieers.com>\nGNU General Public License";
		string desc = "A tray applet for configuring your proxy.";
		string [] authors = new string[] { "Dag Wieers <dag@wieers.com>" };
		Gnome.About about = new Gnome.About(NAME, VERSION, copy, desc, authors, null, null, logo);
		about.Icon = ICON;

		about.Show ();
		about.Deiconify();	// Bring up to front
		System.GC.SuppressFinalize (about);
	}

	private void on_mquit_activate(object o, EventArgs args) {
		Application.Quit();
	}
}
